/*

  The contents of this file are subject to the Mozilla Public License Version
  1.1 (the "License"); you may not use this file except in compliance with
  the License. You may obtain a copy of the License at

           http://www.mozilla.org/MPL/

  Software distributed under the License is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
  for the specific language governing rights and limitations under the License.

  The Original Code is ActionScriptFacile Framework.

  The Initial Developer of the Original Code is
  Matthieu  - http://www.actionscript-facile.com
  Portions created by the Initial Developer are Copyright (C) 2010
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

*/

package application.app
{
	import application.app.controller.SoundActionCommand;
	import application.app.controller.SoundOnOffCommand;
	import application.app.controller.GameOverCommand;
	import application.app.controller.LinkMenuCommand;
	import application.app.controller.StartGameCommand;
	import flash.display.Stage;
	import flash.display.Sprite;


	import application.app.controller.ApplicationStartupCommand;
	import org.puremvc.as3.interfaces.IFacade;
	import org.puremvc.as3.patterns.facade.Facade;

	
	/**
	 * Exemple de création d'une Application Flash avec PureMCV 
	 * et des Composants du Framework AS3 ActionScript-Facile.
	 * 
	 * @author Matthieu
	 */
	public class ApplicationFacade extends Facade implements IFacade
	{
		// définition des noms des évènements par ApplicationFacade pour excécuter des commandes
		public static const STARTUP:String = "startup";

		public static const START_GAME : String = "onStartGame";		
		public static const RESTART_LEVEL : String = "onRestartLevel";		
		public static const MOVE_MAP : String = "onMoveMap";
		public static const MOVE_BACKGROUND : String = "onMoveBackground";
		
		public static const HEROS_JUMP : String = "onHerosJump";
		public static const HEROS_READY : String = "onHerosReady";
		public static const HEROS_SCORE : String = "onHerosScoreTxtUpdate";
		public static const HEROS_LIFE : String = "onHerosLifeTxtUpdate";
		public static const UPDATE_HEROS_SCORE : String = "onUpdateHerosScore";
		
		public static const UPDATE_LOAD_PROGRESS : String = "onUpdateLoadProgress";

		
		public static const UPDATE_POS_HEROS : String = "onUpdatePositionHeros";
		public static const ENNEMIS_TOUCH_HEROS : String = "onHerosTouchByEnnemis";
		
		// évènements utilisés par les Mediators
		public static const ON_ENTER_FRAME : String = "onEnterFrame";
		public static const APPLI_READY : String = "onAppliReady";
		public static const COMMAND_LINK_MENU:String = "LinkMenuCommand";
		
		public static const COMMAND_START_GAME : String = "onCommandReady";
		public static const COMMAND_GAME_OVER : String = "onCommandGameOver";
		
		public static const COMMAND_SOUND_ON_OFF : String = "onPutSoundOnOff";
		public static const COMMAND_HEROS_ACTION_SOUND : String = "onPutSoundJump";
		
	
	
	
		// La façade concrète de votre application
		public function ApplicationFacade() 
		{
			super();
		}
		
		/**
		 * Retourne l'instance de notre façade concrète
		 * 
		 */
		public static function getInstance():ApplicationFacade
		{
			if (instance == null)	instance = new ApplicationFacade();
		    return instance as ApplicationFacade;
		}
		
			
		/**
		 * Envoie la notification pour démarrer l'application
		 * 
		 */
		public function startup(root:Sprite, stage:Stage ):void
		{
			var oData:Object = new Object();
			oData.root = root;
			oData.stage = stage;
			sendNotification(STARTUP, oData);
		}

		 
		/**
		 * Initialise le contrôleur
		 * Et enregistre les commandes pour que celles-ci s'exécutent à l'envoi des notifications correspondantes. 
		 * 
		 */
		protected override function initializeController():void
		{
			super.initializeController();
		    registerCommand(STARTUP, ApplicationStartupCommand);;
			registerCommand(COMMAND_START_GAME, StartGameCommand);
			registerCommand(COMMAND_LINK_MENU, LinkMenuCommand);
			registerCommand(COMMAND_GAME_OVER, GameOverCommand);
			registerCommand(COMMAND_SOUND_ON_OFF, SoundOnOffCommand);
			registerCommand(COMMAND_HEROS_ACTION_SOUND, SoundActionCommand);
		}
	}
}