package application.app.model
{
	import flash.media.SoundTransform;
	import flash.media.SoundChannel;
	import application.app.ApplicationFacade;
	import flash.events.Event;
	import flash.media.Sound;
	import com.lowke.assetLoader.event.LoaderProgressEvent;
	import com.lowke.assetLoader.AssetLoader;
	import com.demonsters.debugger.MonsterDebugger;

	
	import org.puremvc.as3.interfaces.IProxy;
	import org.puremvc.as3.patterns.proxy.Proxy;

	
	/**
	 * Lance la récupération des données de notre application
	 * pour les envoyer à notre model.
	 * 
	 * @author matth
	 */
	 public class RPGProxySound extends Proxy implements IProxy
    {
		public static const NAME : String = "RPGProxySound";
		public static const HERO_JUMP : String = "jump";
		public static const PIECE_WIN : String = "piece";
		public static const ENNEMI_DEAD : String = "ennemi_dead";
		public static const PLAYER_DEAD : String = "player_dead";
		public static const PLAYER_GAME_OVER : String = "player_gameover";
		public static const PLAYER_WIN : String = "player_win";
		
		
		
		private var _assetLoader : AssetLoader;
		private var oSoundChannel : SoundChannel;
 		private var oSoundTransform : SoundTransform;
		
		/* attribution des sons en fonction des actions du joueur */	
		private var _jump:Sound;
		private var _background:Sound;
		
		private var _ennemiDead:Sound;
		private var _dead:Sound;
		private var _gameover:Sound;
		private var _win:Sound;
		
		private var _piece:Sound;
		private var _wait:Sound;
		
		
		
        public function RPGProxySound()
        {
            super(NAME);
			setupModel();
		}
		
		
		/**
		 * Préparation du modèle pour la gestion du son dans le jeu.
		 * 
		 */
		private function setupModel() : void
		{
			MonsterDebugger.trace(this, "setupModel Sound");
			
			oSoundChannel = new SoundChannel();
			oSoundTransform = new SoundTransform();
			
			// recupere la reference AssetLoader singleton
			_assetLoader = AssetLoader.instance;
			
			// ecouteur pour la progression du chargement
			_assetLoader.addEventListener(LoaderProgressEvent.PROGRESS, onLoadProgress);
			
			
			// creation d'une pile de chargement de tous les fichiers mp3 necessaires au jeu
			_assetLoader.loadFile("mp3-mario/background.mp3", assignId, "_background");
			_assetLoader.loadFile("mp3-mario/jump.mp3", assignId, "_jump");
			_assetLoader.loadFile("mp3-mario/player-dead.mp3", assignId, "_dead");
			_assetLoader.loadFile("mp3-mario/ennemi-dead.mp3", assignId, "_ennemiDead");
			_assetLoader.loadFile("mp3-mario/wait.mp3", assignId, "_wait");
			_assetLoader.loadFile("mp3-mario/gameover.mp3", assignId, "_gameover");
			_assetLoader.loadFile("mp3-mario/win.mp3", assignId, "_win");
			_assetLoader.loadFile("mp3-mario/piece.mp3", assignId, "_piece");
			
		
			// lorsque tous les ficheirs de la pile sont chargés appel de cette fonction
			_assetLoader.whenDone(onLoadComplete);
		}
		
		private function onLoadComplete(event:Event = null):void
		{
			// supprime les écouteurs
			_assetLoader.removeEventListener(LoaderProgressEvent.PROGRESS, onLoadProgress);
			
			sendNotification(ApplicationFacade.UPDATE_LOAD_PROGRESS, "Son : <a href='event:sound' >on/off</a>");
			
			// joue le son sélectionné
			oSoundChannel = _wait.play(0,10);
		}


		/**
		 * Lance le son pendant que le joueur joue.
		 * 
		 */
		public function startBackgroundSound() : void
		{
			oSoundChannel.stop();
			oSoundChannel = _background.play(0,34);
			oSoundChannel.soundTransform = oSoundTransform;
		}
	
		/**
		 * Lance le son pendant que le joueur saute.
		 * 
		 */
		public function startJumpSound() : void
		{
			_jump.play();
		}
		
		
		/**
		 * Lance le son pendant que le joueur saute sur un bonus.
		 * 
		 */
		public function startPieceSound() : void
		{
			_piece.play();
		}
		
		
		/**
		 * Lance le son pendant que le joueur saute sur un ennemi.
		 * 
		 */
		public function startEnnemiDeadSound() : void
		{
			_ennemiDead.play();
		}
		
		
		/**
		 * Lance le son pendant que le joueur saute sur un ennemi.
		 * 
		 */
		public function startPlayerDeadSound() : void
		{
			oSoundChannel.stop();
			oSoundChannel = _dead.play();
			oSoundChannel.addEventListener(Event.SOUND_COMPLETE, onSoundDeadCompleteHandler);
			oSoundChannel.soundTransform = oSoundTransform;
		}


		/**
		 * Lance le son pendant que le joueur saute sur un ennemi.
		 * 
		 */
		public function startPlayerGameOverSound() : void
		{
			oSoundChannel.stop();
			oSoundChannel = _gameover.play();
			oSoundChannel.soundTransform = oSoundTransform;
		}


		/**
		 * Lance le son pendant que le joueur a terminé un niveau.
		 * 
		 */
		public function startPlayerWinSound() : void
		{
			oSoundChannel.stop();
			oSoundChannel = _win.play();
			oSoundChannel.soundTransform = oSoundTransform;
			_piece.play(0, 3);
		}
				
		/**
		 * Lancement du niveau à la fin du son.
		 * 
		 */
		private function onSoundDeadCompleteHandler(event : Event) : void
		{
			oSoundChannel.removeEventListener(Event.SOUND_COMPLETE, onSoundDeadCompleteHandler);
			sendNotification(ApplicationFacade.COMMAND_START_GAME, RPGProxy.RESTART_LEVEL);// démarre une nouvelle partie
		}
		
		/**
		 * Arrête la lecture de tous les sons du jeu.
		 * 
		 */
		public function stopAllSound() : void
		{
			oSoundTransform.volume = 0;
			oSoundChannel.soundTransform = oSoundTransform;
		}
		
		
		/**
		 * Relance la lecture de tous les sons du jeu.
		 * 
		 */
		public function startAllSound() : void
		{
			oSoundTransform.volume = 1;
			oSoundChannel.soundTransform = oSoundTransform;
		}
		
		
		/**
		 * Progression du chargement en cours.
		 * A destination d'une vue.
		 * 
		 */
		private function onLoadProgress(event:LoaderProgressEvent):void
		{
			var sTxt:String = "Chargement : " + Math.round(event.percentDone*100) + "%";
			//MonsterDebugger.trace(this, sTxt);	
			sendNotification(ApplicationFacade.UPDATE_LOAD_PROGRESS, sTxt);
		}
		
		
		/**
		 * Association de la classe sound en fonction du mp3 chargé.
		 * 
		 */
		private function assignId(asset:*, memberVariable:String):void 
		{
			if (! asset)
				MonsterDebugger.trace(this, "Erreur de chargement pour l'élément " + memberVariable + " !!!");
			
			this[memberVariable] = asset;// association de l'élément à une variable
		}
		
		
    }

}
