package application.app.view
{
	import flash.events.Event;
	import flash.events.ContextMenuEvent;
	import flash.display.Sprite;
	
	import application.app.ApplicationFacade;
	
	import flash.ui.ContextMenuItem;
	import flash.ui.ContextMenu;
	
	import org.puremvc.as3.interfaces.INotification;
	import org.puremvc.as3.interfaces.IMediator;
	import org.puremvc.as3.patterns.mediator.Mediator;
	
	
	
	/**
	 * ApplicationMediator est le médiateur de notre vue principale.
	 * Il est chargé d'enregistrer tous les autres médiateurs de notre application.
	 * Il sera à l'écoute d'une notification, VIEW_MENU
	 * et lorsqu'il recevra cette notification il changera l'index du viewstack 
	 * pour afficher la vue contenant notre composant MenuView.
	 * 
	 * Il dispose aussi d'un getter permettant de récupérer l'instance de notre mxml principal. 
	 * 
	 * @author matth
	 */
	public class ApplicationMediator extends Mediator implements IMediator
    {
        public static const NAME:String = "ApplicationMediator";
        		
        private var oMenu : ContextMenu;
        private var aLinkMenu : Array;
 
        public function ApplicationMediator(viewComponent:Object)
        {
			super(NAME, viewComponent);
			
			setupAppli();
        }
 
        override public function listNotificationInterests():Array
        {
            return [];
        }
 
 
        override public function handleNotification(note:INotification):void
        {
            /*switch (note.getName())
            {
                case ApplicationFacade.VIEW_MENU:
                	break;
            }*/
        }
 
 		/**
 		 * Récupère le display object principal de la scène.
 		 * 
 		 */
        public function get root():Sprite
        {
            return viewComponent as Sprite;
        }

 
 		/**
		 * Initialisation de l'application.
		 * Le menu contextuel.
		 * 
		 */
		private function setupAppli() : void
		{
			// position des vues
			var viewHeros:Sprite = new Sprite();
			var viewBackground:Sprite = new Sprite();
			var viewBorder:Sprite = new Sprite();
			var viewMap:Sprite = new Sprite();
			var viewGagne:Sprite = new Sprite();
			var viewGameOver:Sprite = new Sprite();
			
			root.addChild(viewBackground);
			root.addChild(viewMap);
			root.addChild(viewHeros);
			root.addChild(viewBorder);
			root.addChild(viewGagne);
			root.addChild(viewGameOver);
			
			// création des vues
			facade.registerMediator(new ViewBackground( viewBackground ));
			facade.registerMediator(new ViewBorder( viewBorder ));
			facade.registerMediator(new ViewMap( viewMap ));
			facade.registerMediator(new ViewHeros( viewHeros ));
			facade.registerMediator(new ViewGagne( viewGagne ));
			facade.registerMediator(new ViewGameOver( viewGameOver ));
			
			// le menu contextuel
			aLinkMenu = new Array();
			oMenu = new ContextMenu();
			oMenu.hideBuiltInItems();
			addItem("Méthode AS3 Facile", "http://www.formation-actionscript.com");
			addItem("Copyright ActionScript Facile", "http://www.actionscript-facile.com");
			root.contextMenu = oMenu;
			oMenu = null;
			
			sendNotification(ApplicationFacade.APPLI_READY);// informe toutes les vues que l'application est prete	
		}
		
		/**
		 * Enregistre un nouveau lien dans le menu.
		 * 
		 */
		private function addItem(label : String, url : String) : void
		{
			var item : ContextMenuItem = new ContextMenuItem(label);
			item.addEventListener(ContextMenuEvent.MENU_ITEM_SELECT, _onItemSelectedHandler);
			oMenu.customItems.push(item);
			
			var oDataLinK:Object = new Object();
			oDataLinK.label = label;
			oDataLinK.url = url;
			aLinkMenu.push(oDataLinK);
		}
		
		/**
		 * Appel du lien.
		 * 
		 */
		private function _onItemSelectedHandler(event : Event) : void
		{
			var oLinkMenu:ContextMenuItem = ContextMenuItem(event.target);
			
			for (var i : int = 0; i < aLinkMenu.length; i++)
			{
				if(oLinkMenu.caption == aLinkMenu[i].label)
				{
					sendNotification(ApplicationFacade.COMMAND_LINK_MENU, aLinkMenu[i].url );
					break;
				}
			}
			
		}
		
		
    }

}
